@@ -82,6 +82,7 @@ static const char* KEY_SHOW_WEEK     = "show_week_numbers";
 static const char *clock_config_tools [] = {
 	"system-config-date",
 	"redhat-config-date",
+        "/sbin/yast2 timezone",
 	"time-admin",
 };
 
@@ -372,17 +373,25 @@ update_clock (ClockData * cd)
 
         gtk_widget_queue_resize (cd->toggle);
 
-	/* Show date in tooltip */
-	loc = g_locale_from_utf8 (_("%A %B %d"), -1, NULL, NULL, NULL);
-	if (!loc)
-		strcpy (date, "???");
-	else if (strftime (date, sizeof (date), loc, tm) <= 0)
-		strcpy (date, "???");
-	g_free (loc);
+        if (!cd->showdate) {
+                /* Show date in tooltip */
+                loc = g_locale_from_utf8 (_("%A %B %d"), -1, NULL, NULL, NULL);
+                if (!loc)
+                        strcpy (date, "???");
+                else if (strftime (date, sizeof (date), loc, tm) <= 0)
+                        strcpy (date, "???");
+                g_free (loc);
 
-	utf8 = g_locale_to_utf8 (date, -1, NULL, NULL, NULL);
-	set_tooltip (cd->applet, cd->toggle, utf8);
-	g_free (utf8);
+                utf8 = g_locale_to_utf8 (date, -1, NULL, NULL, NULL);
+                set_tooltip (cd->applet, cd->toggle, utf8);
+                g_free (utf8);
+        } else {
+#ifdef HAVE_LIBECAL
+                set_tooltip (cd->applet, cd->toggle, _("Click to view your appointments and tasks"));
+#else
+                set_tooltip (cd->applet, cd->toggle, _("Click to view month calendar"));
+#endif
+        }
 }
 
 static void
@@ -1776,25 +1785,30 @@ try_config_tool (GdkScreen  *screen,
 {
 	GtkWidget *dialog;
 	GError    *err;
-	char      *argv[2];
-	char      *app;
+	char     **argv;
+        char      *path;
 
-	g_return_val_if_fail (tool != NULL, FALSE);
+        if (!tool || tool[0] == '\0')
+                return FALSE;
 
-	app = g_find_program_in_path (tool);
+        if (!g_shell_parse_argv (tool, NULL, &argv, NULL))
+                return FALSE;
 
-	if (!app)
-		return FALSE;
-		
-	argv [0] = app;
-	argv [1] = NULL;		
+        if (!(path = g_find_program_in_path (argv [0]))) {
+                g_strfreev (argv);
+                return FALSE;
+        }
+
+        g_free (path);
 
 	err = NULL;
 	if (gdk_spawn_on_screen (screen, NULL, argv, NULL, 0, NULL, NULL, NULL, &err)) {
-		g_free (app);
+                g_strfreev (argv);
 		return TRUE;
 	}
-			
+
+        g_strfreev (argv);
+
 	dialog = gtk_message_dialog_new (NULL,
 					 GTK_DIALOG_DESTROY_WITH_PARENT,
 					 GTK_MESSAGE_ERROR,
@@ -1809,8 +1823,6 @@ try_config_tool (GdkScreen  *screen,
 	gtk_window_set_screen (GTK_WINDOW (dialog), screen);
 			
 	gtk_widget_show_all (dialog);			
-
-	g_free (app);
 		
 	return TRUE;
 }
@@ -1826,8 +1838,7 @@ config_date (BonoboUIComponent *uic,
 
 	screen = gtk_widget_get_screen (cd->applet);
 
-	if (cd->config_tool && cd->config_tool[0] &&
-            try_config_tool (screen, cd->config_tool))
+	if (try_config_tool (screen, cd->config_tool))
 		return;
 	
 	for (i = 0; i < G_N_ELEMENTS (clock_config_tools); i++)
@@ -1935,6 +1946,30 @@ gmt_time_changed (GConfClient  *client,
 	refresh_clock_timeout (clock);
 }
 
+static gboolean
+check_config_tool_command (const char *config_tool)
+{
+        char **argv;
+        char  *path;
+
+        if (!config_tool || config_tool[0] == '\0')
+                return FALSE;
+
+        argv = NULL;
+        if (!g_shell_parse_argv (config_tool, NULL, &argv, NULL))
+                return FALSE;
+
+        if (!(path = g_find_program_in_path (argv [0]))) {
+                g_strfreev (argv);
+                return FALSE;
+        }
+
+        g_free (path);
+        g_strfreev (argv);
+
+        return TRUE;
+}
+
 static void
 config_tool_changed (GConfClient  *client,
                      guint         cnxn_id,
@@ -1948,8 +1983,10 @@ config_tool_changed (GConfClient  *client,
 
 	value = gconf_value_get_string (entry->value);
 
-        g_free (clock->config_tool);
-	clock->config_tool = g_strdup (value);
+        if (check_config_tool_command (value)) {
+                g_free (clock->config_tool);
+                clock->config_tool = g_strdup (value);
+        }
 }
 
 static void
@@ -2122,7 +2159,6 @@ fill_clock_applet (PanelApplet *applet)
 	GError            *error;
         int                format;
 	char              *format_str;
-	char              *config_tool;
 	
 	panel_applet_add_preferences (applet, "/schemas/apps/clock_applet/prefs", NULL);
 	panel_applet_set_flags (applet, PANEL_APPLET_EXPAND_MINOR);
@@ -2218,26 +2254,26 @@ fill_clock_applet (PanelApplet *applet)
 					      NULL);
 	}
 
-        config_tool = NULL;
-	if (cd->config_tool && cd->config_tool [0])
-		config_tool = g_find_program_in_path (cd->config_tool);
+        if (!check_config_tool_command (cd->config_tool)) {
+                g_free (cd->config_tool);
+                cd->config_tool = NULL;
+        }
 
-        if (!config_tool) {
+        if (!cd->config_tool) {
                 int i;
 
                 for (i = 0; i < G_N_ELEMENTS (clock_config_tools); i++)
-                        if ((config_tool = g_find_program_in_path (clock_config_tools [i])))
+                        if (check_config_tool_command (clock_config_tools [i])) {
+                                cd->config_tool = g_strdup (clock_config_tools [i]);
                                 break;
+                        }
         }
 
-	if (!config_tool)
-		bonobo_ui_component_set_prop (popup_component,
-					      "/commands/ClockConfig",
-					      "hidden", "1",
-					      NULL);
-        
-        g_free (config_tool);
-	
+        bonobo_ui_component_set_prop (popup_component,
+                                      "/commands/ClockConfig",
+                                      "hidden", cd->config_tool ? "0" : "1",
+                                      NULL);
+
 	return TRUE;
 }
 
