@@ -21,22 +21,25 @@
  * 02111-1307, USA.
  */
 
+#define WNCK_I_KNOW_THIS_IS_UNSTABLE 1
+
+#ifdef HAVE_CONFIG_H
 #include <config.h>
-#include "showdesktop.h"
-#include <gtk/gtkaboutdialog.h>
+#endif
+
+#include <gtk/gtkicontheme.h>
 #include <gtk/gtktogglebutton.h>
-#include <gtk/gtktooltips.h>
 #include <gtk/gtkimage.h>
 #include <gtk/gtkstock.h>
 #include <gtk/gtkmessagedialog.h>
 #include <gtk/gtkdnd.h>
-#include <libgnomeui/gnome-help.h>
-#include <libgnomeui/gnome-icon-theme.h>
-#define WNCK_I_KNOW_THIS_IS_UNSTABLE
-#include <libwnck/screen.h>
-#include "wncklet.h"
 #include <gdk/gdkx.h>
 
+#include <libwnck/screen.h>
+
+#include "wncklet.h"
+#include "showdesktop.h"
+
 #include <string.h>
 
 #define TIMEOUT_ACTIVATE 1000
@@ -47,7 +50,6 @@ typedef struct {
         GtkWidget *applet;
         GtkWidget *button;
         GtkWidget *image;
-        GdkPixbuf *icon;
         GtkWidget *about_dialog;
 
         PanelAppletOrient orient;
@@ -58,7 +60,7 @@ typedef struct {
         guint showing_desktop : 1;
         guint button_activate;
 
-	GnomeIconTheme *icon_theme;
+	GtkIconTheme *icon_theme;
 } ShowDesktopData;
 
 static void display_help_dialog  (BonoboUIComponent *uic,
@@ -72,35 +74,20 @@ static void update_icon           (ShowDesktopData *sdd);
 static void update_button_state   (ShowDesktopData *sdd);
 static void update_button_display (ShowDesktopData *sdd);
 
+static void theme_changed_callback        (GtkIconTheme    *icon_theme,
+					   ShowDesktopData *sdd);
+
 static void button_toggled_callback       (GtkWidget       *button,
                                            ShowDesktopData *sdd);
 static void show_desktop_changed_callback (WnckScreen      *screen,
                                            ShowDesktopData *sdd);
 
-static void
-set_tooltip (GtkWidget  *widget,
-             const char *tip)
-{
-        GtkTooltips *tooltips;
-
-        tooltips = g_object_get_data (G_OBJECT (widget), "tooltips");
-        if (!tooltips) {
-                tooltips = gtk_tooltips_new ();
-                g_object_ref (tooltips);
-                gtk_object_sink (GTK_OBJECT (tooltips));
-                g_object_set_data_full (G_OBJECT (widget), "tooltips", tooltips,
-                                        (GDestroyNotify) g_object_unref);
-        }
-
-        gtk_tooltips_set_tip (tooltips, widget, tip, NULL);
-}
-
 /* this is when the panel orientation changes */
 
 static void
 applet_change_orient (PanelApplet       *applet,
                       PanelAppletOrient  orient,
-                      ShowDesktopData         *sdd)
+                      ShowDesktopData   *sdd)
 {
         GtkOrientation new_orient;
 
@@ -132,30 +119,8 @@ applet_change_background (PanelApplet               *applet,
                           GdkPixmap                 *pixmap,
                           ShowDesktopData           *sdd)
 {
-	GtkRcStyle *rc_style;
-	GtkStyle   *style;
-
-	/* reset style */
-	gtk_widget_set_style (GTK_WIDGET (sdd->applet), NULL);
-	rc_style = gtk_rc_style_new ();
-	gtk_widget_modify_style (GTK_WIDGET (sdd->applet), rc_style);
-	g_object_unref (rc_style);
-
-	switch (type) {
-	case PANEL_NO_BACKGROUND:
-		break;
-	case PANEL_COLOR_BACKGROUND:
-		gtk_widget_modify_bg (GTK_WIDGET (sdd->applet),
-				      GTK_STATE_NORMAL, color);
-		break;
-	case PANEL_PIXMAP_BACKGROUND:
-		style = gtk_style_copy (GTK_WIDGET (sdd->applet)->style);
-		if (style->bg_pixmap[GTK_STATE_NORMAL])
-			g_object_unref (style->bg_pixmap[GTK_STATE_NORMAL]);
-		style->bg_pixmap[GTK_STATE_NORMAL] = g_object_ref (pixmap);
-		gtk_widget_set_style (GTK_WIDGET (sdd->applet), style);
-		break;
-	}
+	wncklet_change_background (GTK_WIDGET (sdd->applet), type,
+				   color, pixmap);
 }
 
 /* this is when the panel size changes */
@@ -186,46 +151,66 @@ static void
 update_icon (ShowDesktopData *sdd)
 {
         int width, height;
+        GdkPixbuf *icon;
         GdkPixbuf *scaled;
-        double aspect;
         int icon_size;
+	GError *error;
 
-        if (sdd->icon == NULL)
-                return;
+	if (!sdd->icon_theme)
+		return;
 
-        width = gdk_pixbuf_get_width (sdd->icon);
-        height = gdk_pixbuf_get_height (sdd->icon);
+#define SPACE_FOR_BUTTON_BORDER 4
+	icon_size = sdd->size - SPACE_FOR_BUTTON_BORDER;
 
-        aspect = (double) width / (double) height;
+	error = NULL;
+	icon = gtk_icon_theme_load_icon (sdd->icon_theme,
+					 "gnome-fs-desktop",
+					 icon_size, 0, &error);
 
-#define SPACE_FOR_BUTTON_BORDER 3
-        icon_size = sdd->size - SPACE_FOR_BUTTON_BORDER;
+	if (icon == NULL) {
+		g_printerr (_("Failed to load %s: %s\n"), "gnome-fs-desktop",
+			    error ? error->message : _("Icon not found"));
+		if (error) {
+			g_error_free (error);
+			error = NULL;
+		}
+
+		gtk_image_set_from_stock (GTK_IMAGE (sdd->image),
+					  GTK_STOCK_MISSING_IMAGE,
+					  GTK_ICON_SIZE_SMALL_TOOLBAR);
+		return;
+	}
+
+        width = gdk_pixbuf_get_width (icon);
+        height = gdk_pixbuf_get_height (icon);
 
         scaled = NULL;
 
         /* Make it fit on the given panel */
         switch (sdd->orient) {
         case GTK_ORIENTATION_HORIZONTAL:
-                width = icon_size * aspect;
+                width = (icon_size * width) / height;
                 height = icon_size;
                 break;
         case GTK_ORIENTATION_VERTICAL:
-                height = icon_size / aspect;
+                height = (icon_size * height) / width;
                 width = icon_size;
                 break;
         }
 
-        scaled = gdk_pixbuf_scale_simple (sdd->icon,
+        scaled = gdk_pixbuf_scale_simple (icon,
                                           width, height,
                                           GDK_INTERP_BILINEAR);
 
-        if (scaled == NULL)
-                return;
+        if (scaled != NULL) {
+		gtk_image_set_from_pixbuf (GTK_IMAGE (sdd->image),
+					   scaled);
+		g_object_unref (scaled);
+	} else
+		gtk_image_set_from_pixbuf (GTK_IMAGE (sdd->image),
+					   icon);
 
-        gtk_image_set_from_pixbuf (GTK_IMAGE (sdd->image),
-                                   scaled);
-
-        g_object_unref (G_OBJECT (scaled));
+        g_object_unref (icon);
 }
 
 static const BonoboUIVerb show_desktop_menu_verbs [] = {
@@ -241,11 +226,11 @@ static void
 update_button_display (ShowDesktopData *sdd)
 {
         if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (sdd->button))) {
-                set_tooltip (sdd->button,
-                             _("Click here to restore hidden windows."));
+                wncklet_set_tooltip (sdd->button,
+				     _("Click here to restore hidden windows."));
         } else {
-                set_tooltip (sdd->button,
-                             _("Click here to hide all windows and show the desktop."));
+                wncklet_set_tooltip (sdd->button,
+				     _("Click here to hide all windows and show the desktop."));
         }
 }
 
@@ -279,9 +264,6 @@ static void
 applet_destroyed (GtkWidget       *applet,
                   ShowDesktopData *sdd)
 {
-        g_object_unref (G_OBJECT (sdd->icon));
-	g_object_unref (sdd->icon_theme);
-
 	if (sdd->about_dialog) {
 		gtk_widget_destroy (sdd->about_dialog);
 		sdd->about_dialog =  NULL;
@@ -292,6 +274,19 @@ applet_destroyed (GtkWidget       *applet,
 		sdd->button_activate = 0;
 	}
 
+	if (sdd->wnck_screen != NULL) {
+		g_signal_handlers_disconnect_by_func (sdd->wnck_screen,
+						      show_desktop_changed_callback,
+						      sdd);
+		sdd->wnck_screen = NULL;
+	}
+
+	if (sdd->icon_theme != NULL) {
+		g_signal_handlers_disconnect_by_func (sdd->icon_theme,
+						      theme_changed_callback,
+						      sdd);
+		sdd->icon_theme = NULL;
+	}
 
         g_free (sdd);
 }
@@ -349,13 +344,60 @@ button_drag_motion (GtkWidget          *widget,
 	return TRUE;
 }
 
+static void 
+show_desktop_applet_realized (PanelApplet *applet, 
+			      gpointer     data)
+{
+	ShowDesktopData *sdd;
+	GdkScreen       *screen;
+	
+	sdd = (ShowDesktopData *) data;
+
+	if (sdd->wnck_screen != NULL)
+		g_signal_handlers_disconnect_by_func (sdd->wnck_screen,
+						      show_desktop_changed_callback,
+						      sdd);
+
+	if (sdd->icon_theme != NULL)
+		g_signal_handlers_disconnect_by_func (sdd->icon_theme,
+						      theme_changed_callback,
+						      sdd);
+
+	screen = gtk_widget_get_screen (sdd->applet);
+	sdd->wnck_screen = wnck_screen_get (gdk_screen_get_number (screen));
+
+	if (sdd->wnck_screen != NULL)
+		wncklet_connect_while_alive (sdd->wnck_screen,
+					     "showing_desktop_changed",
+					     G_CALLBACK (show_desktop_changed_callback),
+					     sdd,
+					     sdd->applet);
+	else
+		g_warning ("Could not get WnckScreen!");
+
+        show_desktop_changed_callback (sdd->wnck_screen, sdd);
+
+	sdd->icon_theme = gtk_icon_theme_get_for_screen (screen);
+	wncklet_connect_while_alive (sdd->icon_theme, "changed",
+				     G_CALLBACK (theme_changed_callback),
+				     sdd,
+				     sdd->applet);
+
+        update_icon (sdd);
+}
+
+static void
+theme_changed_callback (GtkIconTheme    *icon_theme,
+			ShowDesktopData *sdd)
+{
+	update_icon (sdd);
+}
+
 gboolean
 show_desktop_applet_fill (PanelApplet *applet)
 {
         ShowDesktopData *sdd;
-        char *file;
-        GError *error;
-	AtkObject *atk_obj;
+	AtkObject       *atk_obj;
 
 	panel_applet_set_flags (applet, PANEL_APPLET_EXPAND_MINOR);
 
@@ -363,32 +405,7 @@ show_desktop_applet_fill (PanelApplet *applet)
 
         sdd->applet = GTK_WIDGET (applet);
 
-	sdd->icon_theme = gnome_icon_theme_new ();
-	gnome_icon_theme_set_allow_svg (sdd->icon_theme, TRUE);
-	file = gnome_icon_theme_lookup_icon (sdd->icon_theme,
-					     "gnome-fs-desktop", 48,
-					     NULL, NULL);
-        error = NULL;
-        if (file) {
-                sdd->icon = gdk_pixbuf_new_from_file (file, &error);
-                g_free (file);
-		file = NULL;
-        }
-
-        if (sdd->icon == NULL) {
-                g_printerr (_("Failed to load %s: %s\n"),
-                            file ? file : "gnome-fs-desktop",
-			    error ? error->message : _("File not found"));
-                if (error)
-                        g_error_free (error);
-        }
-
-        if (sdd->icon) {
-                sdd->image = gtk_image_new_from_pixbuf (sdd->icon);
-        } else {
-                sdd->image = gtk_image_new_from_stock (GTK_STOCK_MISSING_IMAGE,
-                                                       GTK_ICON_SIZE_SMALL_TOOLBAR);
-        }
+	sdd->image = gtk_image_new ();
 
         switch (panel_applet_get_orient (applet)) {
         case PANEL_APPLET_ORIENT_LEFT:
@@ -404,7 +421,8 @@ show_desktop_applet_fill (PanelApplet *applet)
 
         sdd->size = panel_applet_get_size (PANEL_APPLET (sdd->applet));
 
-        update_icon (sdd);
+	g_signal_connect (G_OBJECT (sdd->applet), "realize",
+			  G_CALLBACK (show_desktop_applet_realized), sdd);
 
         sdd->button = gtk_toggle_button_new ();
 	atk_obj = gtk_widget_get_accessible (sdd->button);
@@ -419,19 +437,6 @@ show_desktop_applet_fill (PanelApplet *applet)
         gtk_container_add (GTK_CONTAINER (sdd->button), sdd->image);
         gtk_container_add (GTK_CONTAINER (sdd->applet), sdd->button);
 
-        update_button_state (sdd);
-
-        sdd->wnck_screen =
-                wnck_screen_get (gdk_screen_get_number (gtk_widget_get_screen (sdd->applet)));
-
-        if (sdd->wnck_screen != NULL)
-		wncklet_connect_while_alive (sdd->wnck_screen, "showing_desktop_changed",
-					     G_CALLBACK (show_desktop_changed_callback),
-					     sdd,
-					     sdd->applet);
-        else
-                g_warning ("Could not get WnckScreen!");
-
         /* FIXME: Update this comment. */
         /* we have to bind change_orient before we do applet_widget_add
            since we need to get an initial change_orient signal to set our
@@ -483,32 +488,8 @@ display_help_dialog (BonoboUIComponent *uic,
                      ShowDesktopData         *sdd,
                      const gchar       *verbname)
 {
-        GError *error = NULL;
-
-        gnome_help_display_desktop_on_screen (
-                NULL, "user-guide", "user-guide.xml", "gospanel-564",
-                gtk_widget_get_screen (sdd->applet),
-		&error);
-
-        if (error) {
-                GtkWidget *dialog;
-                dialog = gtk_message_dialog_new (NULL,
-                                                 GTK_DIALOG_MODAL,
-                                                 GTK_MESSAGE_ERROR,
-                                                 GTK_BUTTONS_OK,
-                                                 _("There was an error displaying help: %s"),
-                                                 error->message);
-
-                g_signal_connect (G_OBJECT (dialog), "response",
-                                  G_CALLBACK (gtk_widget_destroy),
-                                  NULL);
-
-                gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
-                gtk_window_set_screen (GTK_WINDOW (dialog),
-                                       gtk_widget_get_screen (sdd->applet));
-                gtk_widget_show (dialog);
-                g_error_free (error);
-        }
+	wncklet_display_help (sdd->applet, "user-guide",
+			      "user-guide.xml", "gospanel-564");
 }
 
 static void
@@ -528,36 +509,16 @@ display_about_dialog (BonoboUIComponent *uic,
         /* Translator credits */
         const char *translator_credits = _("translator-credits");
 
-        if (sdd->about_dialog) {
-                gtk_window_set_screen (GTK_WINDOW (sdd->about_dialog),
-                                       gtk_widget_get_screen (sdd->applet));
-                gtk_window_present (GTK_WINDOW (sdd->about_dialog));
-                return;
-        }
-
-        sdd->about_dialog = gtk_about_dialog_new ();
-        g_object_set (sdd->about_dialog,
-                      "name",  _("Show Desktop Button"),
-                      "version", VERSION,
-                      "copyright", "Copyright \xc2\xa9 2002 Red Hat, Inc.",
-                      "comments", _("This button lets you hide all windows and show the desktop."),
-                      "authors", authors,
-                      "documenters", documenters,
-                      "translator_credits", strcmp (translator_credits, "translator-credits") != 0 ? translator_credits : NULL,
-                      "logo_icon_name", "gnome-fs-desktop",
-                      NULL);
-
-        gtk_window_set_wmclass (GTK_WINDOW (sdd->about_dialog), "show-desktop", "show-desktop");
-        gtk_window_set_screen (GTK_WINDOW (sdd->about_dialog),
-                               gtk_widget_get_screen (sdd->applet));
-
-	gtk_window_set_icon_name (GTK_WINDOW (sdd->about_dialog),
-				  "gnome-fs-desktop"); 
-
-        g_signal_connect (G_OBJECT(sdd->about_dialog), "destroy",
-                          (GCallback)gtk_widget_destroyed, &sdd->about_dialog);
-
-        gtk_widget_show (sdd->about_dialog);
+	wncklet_display_about (sdd->applet, &sdd->about_dialog,
+			       _("Show Desktop Button"),
+			       "Copyright \xc2\xa9 2002 Red Hat, Inc.",
+			       _("This button lets you hide all windows and show the desktop."),
+			       authors,
+			       documenters,
+			       translator_credits,
+			       "gnome-fs-desktop",
+			       "show-desktop",
+			       "show-desktop");
 }
 
 static void
