@@ -21,6 +21,7 @@
 static GdkPixbuf * get_missing (GtkIconTheme *icon_theme,
 				int           preffered_size);
 static void button_widget_icon_theme_changed (ButtonWidget *button);
+static void button_widget_reload_pixbuf (ButtonWidget *button);
 
 enum {
 	PROP_0,
@@ -138,6 +139,7 @@ button_widget_realize(GtkWidget *widget)
 				 button,
 				 G_CONNECT_SWAPPED);
 
+	button_widget_reload_pixbuf (BUTTON_WIDGET (widget));
 }
 
 static void
@@ -189,6 +191,8 @@ button_widget_reload_pixbuf (ButtonWidget *button)
 		button->pixbuf = panel_load_icon (button->icon_theme,
 						  button->filename,
 						  button->size,
+						  button->orientation & PANEL_VERTICAL_MASK   ? button->size : -1,
+						  button->orientation & PANEL_HORIZONTAL_MASK ? button->size : -1,
 						  &error);
 		if (error) {
 			panel_error_dialog (gdk_screen_get_default (),
@@ -504,6 +508,8 @@ button_widget_size_allocate (GtkWidget     *widget,
 	else
 		size = allocation->width;
 
+	size = CLAMP (size, 1, 48);
+
 	if (button_widget->size != size) {
 		button_widget->size = size;
 
