@@ -29,6 +29,9 @@
 #include <glib/gi18n.h>
 #include <gdk/gdkkeysyms.h>
 #include <libgnome/gnome-desktop-item.h>
+#include <libgnomevfs/gnome-vfs-result.h>
+#include <libgnomevfs/gnome-vfs-uri.h>
+#include <libgnomevfs/gnome-vfs-xfer.h>
 #include <gconf/gconf-client.h>
 
 #include "launcher.h"
@@ -41,16 +44,18 @@
 #include "panel-recent.h"
 #include "panel-profile.h"
 #include "panel-menu-button.h"
+#include "panel-menu-items.h"
 #include "panel-globals.h"
 #include "panel-run-dialog.h"
 #include "panel-lockdown.h"
 
 typedef struct {
-	GtkWidget   *pixmap;
-	const char  *stock_id;
-	char        *image;
-	char        *fallback_image;
-	GtkIconSize  icon_size;
+	GtkWidget    *pixmap;
+	const char   *stock_id;
+	char         *image;
+	char         *fallback_image;
+	GtkIconTheme *icon_theme;
+	GtkIconSize   icon_size;
 } IconToLoad;
 
 typedef struct {
@@ -67,8 +72,8 @@ static GList *icons_to_add = NULL;
 
 static GSList *image_menu_items = NULL;
 
-static GtkWidget *populate_menu_from_directory (GtkWidget         *menu,
-						MenuTreeDirectory *directory);
+static GtkWidget *populate_menu_from_directory (GtkWidget          *menu,
+						GMenuTreeDirectory *directory);
 
 static void panel_load_menu_image_deferred (GtkWidget   *image_menu_item,
 					    GtkIconSize  icon_size,
@@ -88,6 +93,16 @@ panel_menu_have_icons (void)
 			NULL);
 }
 
+static inline gboolean
+desktop_is_home_dir (void)
+{	
+	return gconf_client_get_bool (
+			panel_gconf_get_client (),
+			"/apps/nautilus/preferences/desktop_is_home_dir",
+			NULL);
+}
+
+
 GtkWidget *
 add_menu_separator (GtkWidget *menu)
 {
@@ -102,42 +117,13 @@ add_menu_separator (GtkWidget *menu)
 }
 
 static void
-activate_app_def (GtkWidget     *menuitem,
-		  MenuTreeEntry *entry)
+activate_app_def (GtkWidget      *menuitem,
+		  GMenuTreeEntry *entry)
 {
-	GnomeDesktopItem *item;
-	GError           *error;
 	const char       *path;
 
-	path = menu_tree_entry_get_desktop_file_path (entry);
-
-	error = NULL;
-	item = gnome_desktop_item_new_from_file (path, 0, &error);
-	if (item) {
-		g_assert (error == NULL);
-
-		panel_ditem_launch (
-			item, NULL, 0, menuitem_to_screen (menuitem), &error);
-		if (error) {
-			panel_error_dialog (menuitem_to_screen (menuitem),
-					    "cannot_launch_entry", TRUE,
-					    _("Cannot launch entry"),
-					    "%s",
-					    error->message);
-
-			g_error_free (error);
-		}
-		gnome_desktop_item_unref (item);
-	} else {
-		g_assert (error != NULL);
-
-		panel_error_dialog (menuitem_to_screen (menuitem),
-				    "cannot_load_entry", TRUE,
-				    _("Cannot load entry"),
-				    "%s",
-				    error->message);
-		g_error_free (error);
-	}
+	path = gmenu_tree_entry_get_desktop_file_path (entry);
+	panel_menu_item_activate_desktop_file (menuitem, path);
 }
 
 PanelWidget *
@@ -186,6 +172,10 @@ setup_menu_panel (GtkWidget *menu)
 
 	panel = menu_get_panel (menu);
 	g_object_set_data (G_OBJECT (menu), "menu_panel", panel);
+
+	if (panel)
+		gtk_menu_set_screen (GTK_MENU (menu),
+				     gtk_widget_get_screen (GTK_WIDGET (panel)));
 }
 
 GdkScreen *
@@ -210,7 +200,7 @@ menus_have_icons_changed (GConfClient *client,
 
 	value = gconf_entry_get_value (entry);
 
-	if (value->type == GCONF_VALUE_BOOL)
+	if (value != NULL && value->type == GCONF_VALUE_BOOL)
 		have_icons = gconf_value_get_bool (value);
 
 	list = g_list_copy (GTK_MENU_SHELL (menu)->children);
@@ -285,7 +275,7 @@ panel_create_menu (void)
 	if (!registered_icon_theme_changer) {
 		registered_icon_theme_changer = TRUE;
 
-		g_signal_connect (panel_icon_theme, "changed",
+		g_signal_connect (gtk_icon_theme_get_default (), "changed",
 				  G_CALLBACK (icon_theme_changed), NULL);
 	}
 	
@@ -309,8 +299,12 @@ create_empty_menu (void)
 
 	retval = panel_create_menu ();
 
-	g_signal_connect (
-		retval, "show", G_CALLBACK (setup_menu_panel), NULL);
+	g_signal_connect (retval, "show", G_CALLBACK (setup_menu_panel), NULL);
+
+	/* intercept all right button clicks makes sure they don't
+	   go to the object itself */
+	g_signal_connect (retval, "button_press_event",
+			  G_CALLBACK (menu_dummy_button_press_event), NULL);
 
 	return retval;
 }
@@ -358,10 +352,11 @@ remove_pixmap_from_loaded (gpointer data, GObject *where_the_object_was)
 }
 
 GdkPixbuf *
-panel_make_menu_icon (const char *icon,
-		      const char *fallback,
-		      int         size,
-		      gboolean   *long_operation)
+panel_make_menu_icon (GtkIconTheme *icon_theme,
+		      const char   *icon,
+		      const char   *fallback,
+		      int           size,
+		      gboolean     *long_operation)
 {
 	GdkPixbuf *pb;
 	char *file, *key;
@@ -371,15 +366,9 @@ panel_make_menu_icon (const char *icon,
 
 	file = NULL;
 	if (icon != NULL)
-		file = gnome_desktop_item_find_icon (panel_icon_theme,
-						     icon,
-						     size /* desired size */,
-						     0 /* flags */);
+		file = panel_find_icon (icon_theme, icon, size);
 	if (file == NULL && fallback != NULL)
-		file = gnome_desktop_item_find_icon (panel_icon_theme,
-						     fallback,
-						     size /* desired size */,
-						     0 /* flags */);
+		file = panel_find_icon (icon_theme, fallback, size);
 
 	if (file == NULL)
 		return NULL;
@@ -571,7 +560,8 @@ load_icons_handler_again:
 
 		gtk_icon_size_lookup (icon->icon_size, NULL, &icon_height);
 
-		pb = panel_make_menu_icon (icon->image,
+		pb = panel_make_menu_icon (icon->icon_theme,
+					   icon->image,
 					   icon->fallback_image,
 					   icon_height,
 					   &long_operation);
@@ -608,8 +598,8 @@ load_icons_handler_again:
 }
 
 static void
-add_app_to_panel (GtkWidget     *item,
-		  MenuTreeEntry *entry)
+add_app_to_panel (GtkWidget      *item,
+		  GMenuTreeEntry *entry)
 {
 	PanelWidget   *panel_widget;
 	PanelToplevel *toplevel;
@@ -624,24 +614,106 @@ add_app_to_panel (GtkWidget     *item,
 
 	panel_launcher_create (toplevel,
 			       position,
-			       menu_tree_entry_get_desktop_file_path (entry));
+			       gmenu_tree_entry_get_desktop_file_path (entry));
 }
 
 
 static void
-add_drawers_from_dir (MenuTreeDirectory *directory,
-		      int                pos,
-		      const char        *toplevel_id)
+add_app_to_desktop (GtkWidget      *item,
+		    GMenuTreeEntry *entry)
+{
+	GnomeVFSURI *vfs_source_uri;
+	GnomeVFSURI *vfs_target_uri;
+	GnomeVFSResult res;
+	char *source_uri;
+	char *target_dir;
+	char *target_uri;
+
+	g_return_if_fail (entry != NULL);
+
+	source_uri = g_filename_to_uri (gmenu_tree_entry_get_desktop_file_path (entry), NULL, NULL);
+	g_return_if_fail (source_uri != NULL);
+
+	vfs_source_uri = gnome_vfs_uri_new (source_uri);
+	g_return_if_fail (vfs_source_uri != NULL);
+
+	if (desktop_is_home_dir ()) {
+		target_dir = g_build_filename (g_get_home_dir (), NULL);
+	} else {
+		target_dir = g_build_filename (g_get_home_dir (), "Desktop", NULL);
+	}
+
+	target_uri = panel_make_unique_uri (target_dir, ".desktop");
+	g_return_if_fail (target_uri != NULL);
+
+	vfs_target_uri = gnome_vfs_uri_new (target_uri);
+	g_return_if_fail (vfs_target_uri != NULL);
+
+	res = gnome_vfs_xfer_uri (vfs_source_uri, vfs_target_uri,
+				  GNOME_VFS_XFER_DEFAULT,
+				  GNOME_VFS_XFER_ERROR_MODE_ABORT,
+				  GNOME_VFS_XFER_OVERWRITE_MODE_ABORT,
+				  NULL, NULL);
+
+	if (res != GNOME_VFS_OK) {
+		g_warning ("Problem while copying launcher to desktop: %s",
+			   gnome_vfs_result_to_string (res));
+	}
+
+	gnome_vfs_uri_unref (vfs_source_uri);
+	gnome_vfs_uri_unref (vfs_target_uri);
+
+	g_free (source_uri);
+	g_free (target_dir);
+	g_free (target_uri);
+}
+
+
+static void add_drawers_from_dir (GMenuTreeDirectory *directory,
+				  int                 pos,
+				  const char         *toplevel_id);
+
+static void
+add_drawers_from_alias (GMenuTreeAlias *alias,
+			const char     *toplevel_id)
+{
+	GMenuTreeItem *aliased_item;
+
+	aliased_item = gmenu_tree_alias_get_item (alias);
+
+	switch (gmenu_tree_item_get_type (aliased_item)) {
+	case GMENU_TREE_ITEM_DIRECTORY:
+		add_drawers_from_dir (GMENU_TREE_DIRECTORY (aliased_item),
+				      G_MAXINT/2,
+				      toplevel_id);
+		break;
+
+	case GMENU_TREE_ITEM_ENTRY:
+		panel_launcher_create_with_id (toplevel_id,
+					       G_MAXINT/2,
+					       gmenu_tree_entry_get_desktop_file_path (GMENU_TREE_ENTRY (aliased_item)));
+		break;
+
+	default:
+		break;
+	}
+
+	gmenu_tree_item_unref (aliased_item);
+}
+
+static void
+add_drawers_from_dir (GMenuTreeDirectory *directory,
+		      int                 pos,
+		      const char         *toplevel_id)
 {
 	const char *name;
 	const char *icon;
-	GSList     *entries;
-	GSList     *subdirs;
+	GSList     *items;
 	GSList     *l;
 	char       *attached_toplevel_id;
 
-	name = menu_tree_directory_get_name (directory);
-	icon = menu_tree_directory_get_icon (directory);
+	name = gmenu_tree_directory_get_name (directory);
+	icon = gmenu_tree_directory_get_icon (directory);
 
 	attached_toplevel_id = panel_drawer_create_with_id (toplevel_id,
 							    pos,
@@ -651,42 +723,50 @@ add_drawers_from_dir (MenuTreeDirectory *directory,
 	if (!attached_toplevel_id)
 		return;
 
-	entries = menu_tree_directory_get_entries (directory);
-	for (l = entries; l; l = l->next) {
-		MenuTreeEntry *entry = l->data;
+	items = gmenu_tree_directory_get_contents (directory);
+	for (l = items; l; l = l->next) {
+		GMenuTreeItem *item = l->data;
 
-		panel_launcher_create_with_id (attached_toplevel_id,
-					       G_MAXINT/2,
-					       menu_tree_entry_get_desktop_file_path (entry));
+		switch (gmenu_tree_item_get_type (item)) {
+		case GMENU_TREE_ITEM_ENTRY:
+			panel_launcher_create_with_id (attached_toplevel_id,
+						       G_MAXINT/2,
+						       gmenu_tree_entry_get_desktop_file_path (GMENU_TREE_ENTRY (item)));
+			break;
 
-		menu_tree_entry_unref (entry);
+		case GMENU_TREE_ITEM_DIRECTORY:
+			add_drawers_from_dir (GMENU_TREE_DIRECTORY (item),
+					      G_MAXINT/2,
+					      attached_toplevel_id);
+			break;
+
+		case GMENU_TREE_ITEM_ALIAS:
+			add_drawers_from_alias (GMENU_TREE_ALIAS (item), attached_toplevel_id);
+			break;
+
+		default:
+			break;
+		}
+
+		gmenu_tree_item_unref (item);
 	}
-	g_slist_free (entries);
 
-	subdirs = menu_tree_directory_get_subdirs (directory);
-	for (l = subdirs; l; l = l->next) {
-		MenuTreeDirectory *subdir = l->data;
-
-		add_drawers_from_dir (subdir, G_MAXINT/2, attached_toplevel_id);
-
-		menu_tree_directory_unref (subdir);
-	}
-	g_slist_free (subdirs);
+	g_slist_free (items);
 
 	g_free (attached_toplevel_id);
 }
 
 static void
-add_menudrawer_to_panel (GtkWidget     *menuitem,
-			 MenuTreeEntry *entry)
+add_menudrawer_to_panel (GtkWidget      *menuitem,
+			 GMenuTreeEntry *entry)
 
 {
-	MenuTreeDirectory *directory;
+	GMenuTreeDirectory *directory;
 	PanelWidget       *panel;
 	PanelData         *pd;
 	int                insertion_pos;
 
-	directory = menu_tree_entry_get_parent (entry);
+	directory = gmenu_tree_item_get_parent (GMENU_TREE_ITEM (entry));
 
 	panel = menu_get_panel (menuitem);
 
@@ -697,37 +777,47 @@ add_menudrawer_to_panel (GtkWidget     *menuitem,
 			      insertion_pos,
 			      panel_profile_get_toplevel_id (panel->toplevel));
 
-	menu_tree_directory_unref (directory);
+	gmenu_tree_item_unref (directory);
 }
 
 static void
-add_menu_to_panel (GtkWidget     *menuitem,
-		   MenuTreeEntry *entry)
+add_menu_to_panel (GtkWidget      *menuitem,
+		   GMenuTreeEntry *entry)
 {
-	MenuTreeDirectory *directory;
-	PanelWidget       *panel;
-	PanelData         *pd;
-	int                insertion_pos;
-	char              *menu_path;
+	GMenuTreeDirectory *directory;
+	PanelWidget        *panel;
+	PanelData          *pd;
+	int                 insertion_pos;
+	char               *menu_path;
+	char               *menu_filename;
 
-	directory = menu_tree_entry_get_parent (entry);
+	menu_filename = g_object_get_data (G_OBJECT (menuitem->parent),
+					   "panel-menu-tree-filename");
+
+	if (!menu_filename) {
+		g_warning ("Cannot find the filename for the menu");
+		return;
+	}
+
+	directory = gmenu_tree_item_get_parent (GMENU_TREE_ITEM (entry));
 
 	panel = menu_get_panel (menuitem);
 
 	pd = g_object_get_data (G_OBJECT (panel->toplevel), "PanelData");
 	insertion_pos = pd ? pd->insertion_pos : -1;
 
-	menu_path = menu_tree_directory_make_path (directory, NULL);
+	menu_path = gmenu_tree_directory_make_path (directory, NULL);
 
 	panel_menu_button_create (panel->toplevel,
 				  insertion_pos,
+				  menu_filename,
 				  menu_path,
 				  TRUE,
-				  menu_tree_directory_get_name (directory));
+				  gmenu_tree_directory_get_name (directory));
 
 	g_free (menu_path);
 
-	menu_tree_directory_unref (directory);
+	gmenu_tree_item_unref (directory);
 }
 
 /*most of this function stolen from the real gtk_menu_popup*/
@@ -786,17 +876,33 @@ static GtkWidget *
 create_item_context_menu (GtkWidget   *item,
 			  PanelWidget *panel_widget)
 {
-	MenuTreeEntry *entry;
-	GtkWidget     *menu;
-	GtkWidget     *submenu;
-	GtkWidget     *menuitem;
-	gboolean       id_lists_writable;
+	GMenuTreeEntry *entry;
+	GtkWidget      *parent;
+	GtkWidget      *menu;
+	GtkWidget      *submenu;
+	GtkWidget      *menuitem;
+	char           *menu_filename;
+	gboolean        id_lists_writable;
 
 	id_lists_writable = panel_profile_id_lists_are_writable ();
 
 	entry = g_object_get_data (G_OBJECT (item), "panel-menu-tree-entry");
-	g_assert (entry != NULL);
+	if (!entry)
+		return NULL;
 
+	parent = item->parent;
+
+	menu_filename = NULL;
+	while (parent) {
+		menu_filename = g_object_get_data (G_OBJECT (parent),
+						   "panel-menu-tree-filename");
+		if (menu_filename)
+			break;
+
+		parent = gtk_menu_get_attach_widget (GTK_MENU (parent))->parent;
+		if (!GTK_IS_MENU (parent))
+			break;
+	}
 
 	menu = create_empty_menu ();
 	g_object_set_data_full (G_OBJECT (item),
@@ -806,8 +912,6 @@ create_item_context_menu (GtkWidget   *item,
 	gtk_object_sink (GTK_OBJECT (menu));
 
 	g_object_set_data (G_OBJECT (menu), "menu_panel", panel_widget);
-
-
 	g_signal_connect (menu, "deactivate",
 			  G_CALLBACK (restore_grabs), item);
 
@@ -822,11 +926,25 @@ create_item_context_menu (GtkWidget   *item,
 	gtk_widget_set_sensitive (menuitem, id_lists_writable);
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
 
+	menuitem = gtk_image_menu_item_new ();
+	setup_menuitem (menuitem,
+			panel_menu_icon_get_size (),
+			NULL,
+			_("Add this launcher to desktop"),
+			FALSE);
+	g_signal_connect (menuitem, "activate",
+			  G_CALLBACK (add_app_to_desktop), entry);
+	gtk_widget_set_sensitive (menuitem, id_lists_writable);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
+
 
 	submenu = create_empty_menu ();
 
 	g_object_set_data (G_OBJECT (submenu), "menu_panel", panel_widget);
-
+	g_object_set_data_full (G_OBJECT (submenu),
+				"panel-menu-tree-filename",
+				g_strdup (menu_filename),
+				(GDestroyNotify) g_free);
 
 	menuitem = gtk_image_menu_item_new ();
 	setup_menuitem (menuitem,
@@ -881,6 +999,9 @@ show_item_menu (GtkWidget      *item,
 	if (!menu)
 		menu = create_item_context_menu (item, panel_widget);
 
+	if (!menu)
+		return FALSE;
+
 	gtk_menu_set_screen (GTK_MENU (menu),
 			     gtk_window_get_screen (GTK_WINDOW (panel_widget->toplevel)));
 
@@ -982,11 +1103,11 @@ drag_data_get_menu_cb (GtkWidget        *widget,
 		       GtkSelectionData *selection_data,
 		       guint             info,
 		       guint             time,
-		       MenuTreeEntry    *entry)
+		       GMenuTreeEntry   *entry)
 {
 	char *uri_list;
 
-	uri_list = g_strconcat (menu_tree_entry_get_desktop_file_path (entry),
+	uri_list = g_strconcat (gmenu_tree_entry_get_desktop_file_path (entry),
 				"\r\n",
 				NULL);
 
@@ -1150,11 +1271,8 @@ setup_internal_applet_drag (GtkWidget             *menuitem,
 			     menu_item_targets, 1,
 			     GDK_ACTION_COPY);
 
-	if (panel_action_get_stock_icon (type)  != NULL)
-		gtk_drag_source_set_icon_stock (menuitem,
-						panel_action_get_stock_icon (type));
 	/* FIXME: waiting for bug #116577
-	else
+	if (panel_action_get_icon_name (type)  != NULL)
 		gtk_drag_source_set_icon_name (GTK_WIDGET (button),
 					panel_action_get_icon_name (type);
 					*/
@@ -1169,11 +1287,11 @@ setup_internal_applet_drag (GtkWidget             *menuitem,
 }
 
 static void
-submenu_to_display (GtkWidget *menu)
+load_submenu_to_display (GtkWidget *menu)
 {
-	MenuTree          *tree;
-	MenuTreeDirectory *directory;
-	const char        *menu_path;
+	GMenuTree          *tree;
+	GMenuTreeDirectory *directory;
+	const char         *menu_path;
 
 	if (!g_object_get_data (G_OBJECT (menu), "panel-menu-needs-loading"))
 		return;
@@ -1185,19 +1303,31 @@ submenu_to_display (GtkWidget *menu)
 		if (!(tree = g_object_get_data (G_OBJECT (menu), "panel-menu-tree")))
 			return;
 
-		directory = menu_tree_get_directory_from_path (tree, menu_path);
+		directory = gmenu_tree_get_directory_from_path (tree, menu_path);
 
 		g_object_set_data_full (G_OBJECT (menu), "panel-menu-tree-directory",
 					directory,
-					(GDestroyNotify) menu_tree_directory_unref);
+					(GDestroyNotify) gmenu_tree_item_unref);
 	}
 
 	if (directory)
 		populate_menu_from_directory (menu, directory);
 }
 
+static gboolean
+submenu_to_display (gpointer data)
+{
+	GtkWidget         *menu;
+
+	menu = GTK_WIDGET (data);
+
+	load_submenu_to_display (menu);
+
+	return FALSE;
+}
+
 static GtkWidget *
-create_fake_menu (MenuTreeDirectory *directory)
+create_fake_menu (GMenuTreeDirectory *directory)
 {	
 	GtkWidget *menu;
 	
@@ -1205,8 +1335,8 @@ create_fake_menu (MenuTreeDirectory *directory)
 
 	g_object_set_data_full (G_OBJECT (menu),
 				"panel-menu-tree-directory",
-				menu_tree_directory_ref (directory),
-				(GDestroyNotify) menu_tree_directory_unref);
+				gmenu_tree_item_ref (directory),
+				(GDestroyNotify) gmenu_tree_item_unref);
 	
 	g_object_set_data (G_OBJECT (menu),
 			   "panel-menu-needs-loading",
@@ -1217,54 +1347,87 @@ create_fake_menu (MenuTreeDirectory *directory)
 			   GUINT_TO_POINTER (TRUE));
 
 	g_signal_connect (menu, "show",
-			  G_CALLBACK (submenu_to_display), NULL);
+			  G_CALLBACK (load_submenu_to_display), NULL);
+	g_idle_add_full (G_PRIORITY_LOW, submenu_to_display, menu, NULL);
 	g_signal_connect (menu, "button_press_event",
 			  G_CALLBACK (menu_dummy_button_press_event), NULL);
 
 	return menu;
 }
 
-static void
-create_submenu (GtkWidget         *menu,
-		MenuTreeDirectory *directory)
+static GtkWidget *
+create_submenu_entry (GtkWidget          *menu,
+		      GMenuTreeDirectory *directory)
 {
 	GtkWidget *menuitem;
-	GtkWidget *submenu;
 
 	menuitem = gtk_image_menu_item_new ();
 
-	g_object_set_data_full (G_OBJECT (menuitem),
-				"panel-menu-tree-directory",
-				menu_tree_directory_ref (directory),
-				(GDestroyNotify) menu_tree_directory_unref);
-
 	panel_load_menu_image_deferred (menuitem,
 					panel_menu_icon_get_size (),
 					NULL,
-					menu_tree_directory_get_icon (directory),
+					gmenu_tree_directory_get_icon (directory),
 					"gnome-fs-directory");
 
 	setup_menuitem (menuitem,
 			panel_menu_icon_get_size (),
 			NULL,
-			menu_tree_directory_get_name (directory),
+			gmenu_tree_directory_get_name (directory),
 			TRUE);
 
-	if (menu_tree_directory_get_comment (directory))
+	if (gmenu_tree_directory_get_comment (directory))
 		gtk_tooltips_set_tip (panel_tooltips,
 				      menuitem,
-				      menu_tree_directory_get_comment (directory),
+				      gmenu_tree_directory_get_comment (directory),
 				      NULL);
 
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
 
-	submenu = create_fake_menu (directory);
-	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem), submenu);
+	return menuitem;
 }
 
 static void
-create_menuitem (GtkWidget     *menu,
-		 MenuTreeEntry *entry)
+create_submenu (GtkWidget          *menu,
+		GMenuTreeDirectory *directory,
+		GMenuTreeDirectory *alias_directory)
+{
+	GtkWidget *menuitem;
+	GtkWidget *submenu;
+
+	if (alias_directory)
+		menuitem = create_submenu_entry (menu, alias_directory);
+	else
+		menuitem = create_submenu_entry (menu, directory);
+	
+	submenu = create_fake_menu (directory);
+
+	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem), submenu);
+}
+
+static void 
+create_header (GtkWidget       *menu,
+	       GMenuTreeHeader *header)
+{
+	GMenuTreeDirectory *directory;
+	GtkWidget          *menuitem;
+
+	directory = gmenu_tree_header_get_directory (header);
+	menuitem = create_submenu_entry (menu, directory);
+	gmenu_tree_item_unref (directory);
+
+	g_object_set_data_full (G_OBJECT (menuitem),
+				"panel-gmenu-tree.header",
+				gmenu_tree_item_ref (header),
+				(GDestroyNotify) gmenu_tree_item_unref);
+
+	g_signal_connect (menuitem, "activate",
+			  G_CALLBACK (gtk_false), NULL);
+}
+
+static void
+create_menuitem (GtkWidget          *menu,
+		 GMenuTreeEntry     *entry,
+		 GMenuTreeDirectory *alias_directory)
 {
 	GtkWidget  *menuitem;
 	
@@ -1272,29 +1435,38 @@ create_menuitem (GtkWidget     *menu,
 
 	g_object_set_data_full (G_OBJECT (menuitem),
 				"panel-menu-tree-entry",
-				menu_tree_entry_ref (entry),
-				(GDestroyNotify) menu_tree_entry_unref);
+				gmenu_tree_item_ref (entry),
+				(GDestroyNotify) gmenu_tree_item_unref);
+
+	if (alias_directory)
+		g_object_set_data_full (G_OBJECT (menuitem),
+					"panel-menu-tree-alias-directory",
+					gmenu_tree_item_ref (alias_directory),
+					(GDestroyNotify) gmenu_tree_item_unref);
 
 	panel_load_menu_image_deferred (menuitem,
 					panel_menu_icon_get_size (),
 					NULL,
-					menu_tree_entry_get_icon (entry),
+					alias_directory ? gmenu_tree_directory_get_icon (alias_directory) :
+							  gmenu_tree_entry_get_icon (entry),
 					NULL);
 
 	setup_menuitem (menuitem,
 			panel_menu_icon_get_size (),
 			NULL,
-			menu_tree_entry_get_name (entry),
+			alias_directory ? gmenu_tree_directory_get_name (alias_directory) :
+					  gmenu_tree_entry_get_name (entry),
 			TRUE);
 
-	if (menu_tree_entry_get_comment (entry))
+	if (gmenu_tree_entry_get_comment (entry))
 		gtk_tooltips_set_tip (panel_tooltips,
 				      menuitem,
-				      menu_tree_entry_get_comment (entry),
+				      alias_directory ? gmenu_tree_directory_get_comment (alias_directory) :
+							gmenu_tree_entry_get_comment (entry),
 				      NULL);
 
-	g_signal_connect (menuitem, "button_press_event",
-			  G_CALLBACK (menuitem_button_press_event), NULL);
+	g_signal_connect_after (menuitem, "button_press_event",
+				G_CALLBACK (menuitem_button_press_event), NULL);
 
 	if (!panel_lockdown_get_locked_down ()) {
 		static GtkTargetEntry menu_item_targets[] = {
@@ -1308,7 +1480,7 @@ create_menuitem (GtkWidget     *menu,
 
 		/* FIXME: waiting for bug #116577
 		gtk_drag_source_set_icon_name (GTK_WIDGET (button),
-					menu_tree_entry_get_icon (entry));
+					gmenu_tree_entry_get_icon (entry));
 		*/
 		g_signal_connect (menuitem, "drag_data_get",
 				  G_CALLBACK (drag_data_get_menu_cb), entry);
@@ -1323,8 +1495,36 @@ create_menuitem (GtkWidget     *menu,
 }
 
 static void
-handle_menu_tree_changed (MenuTree  *tree,
-			  GtkWidget *menu)
+create_menuitem_from_alias (GtkWidget      *menu,
+			    GMenuTreeAlias *alias)
+{
+	GMenuTreeItem *aliased_item;
+
+	aliased_item = gmenu_tree_alias_get_item (alias);
+
+	switch (gmenu_tree_item_get_type (aliased_item)) {
+	case GMENU_TREE_ITEM_DIRECTORY:
+		create_submenu (menu,
+				GMENU_TREE_DIRECTORY (aliased_item),
+				gmenu_tree_alias_get_directory (alias));
+		break;
+
+	case GMENU_TREE_ITEM_ENTRY:
+		create_menuitem (menu,
+				 GMENU_TREE_ENTRY (aliased_item),
+				 gmenu_tree_alias_get_directory (alias));
+		break;
+
+	default:
+		break;
+	}
+
+	gmenu_tree_item_unref (aliased_item);
+}
+
+static void
+handle_gmenu_tree_changed (GMenuTree *tree,
+			   GtkWidget *menu)
 {
 	while (GTK_MENU_SHELL (menu)->children)
                 gtk_widget_destroy (GTK_MENU_SHELL (menu)->children->data);
@@ -1343,11 +1543,11 @@ handle_menu_tree_changed (MenuTree  *tree,
 }
 
 static void
-remove_menu_tree_monitor (GtkWidget *menu,
-			  MenuTree  *tree)
+remove_gmenu_tree_monitor (GtkWidget *menu,
+			  GMenuTree  *tree)
 {
-	menu_tree_remove_monitor (tree,
-				  (MenuTreeChangedFunc) handle_menu_tree_changed,
+	gmenu_tree_remove_monitor (tree,
+				  (GMenuTreeChangedFunc) handle_gmenu_tree_changed,
 				  menu);
 }
 
@@ -1355,17 +1555,22 @@ GtkWidget *
 create_applications_menu (const char *menu_file,
 			  const char *menu_path)
 {
-	MenuTree  *tree;
+	GMenuTree *tree;
 	GtkWidget *menu;
 
 	menu = create_empty_menu ();
 
-	tree = menu_tree_lookup (menu_file);
+	tree = gmenu_tree_lookup (menu_file, GMENU_TREE_FLAGS_NONE);
+
+	g_object_set_data_full (G_OBJECT (menu),
+				"panel-menu-tree-filename",
+				g_strdup (menu_file),
+				(GDestroyNotify) g_free);
 
 	g_object_set_data_full (G_OBJECT (menu),
 				"panel-menu-tree",
-				menu_tree_ref (tree),
-				(GDestroyNotify) menu_tree_unref);
+				gmenu_tree_ref (tree),
+				(GDestroyNotify) gmenu_tree_unref);
 
 	g_object_set_data_full (G_OBJECT (menu),
 				"panel-menu-tree-path",
@@ -1381,61 +1586,72 @@ create_applications_menu (const char *menu_file,
 			   GUINT_TO_POINTER (TRUE));
 
 	g_signal_connect (menu, "show",
-			  G_CALLBACK (submenu_to_display), NULL);
+			  G_CALLBACK (load_submenu_to_display), NULL);
+	g_idle_add_full (G_PRIORITY_LOW, submenu_to_display, menu, NULL);
 	g_signal_connect (menu, "button_press_event",
 			  G_CALLBACK (menu_dummy_button_press_event), NULL);
 
-	menu_tree_add_monitor (tree,
-			       (MenuTreeChangedFunc) handle_menu_tree_changed,
+	gmenu_tree_add_monitor (tree,
+			       (GMenuTreeChangedFunc) handle_gmenu_tree_changed,
 			       menu);
 	g_signal_connect (menu, "destroy",
-			  G_CALLBACK (remove_menu_tree_monitor), tree);
+			  G_CALLBACK (remove_gmenu_tree_monitor), tree);
 
-	menu_tree_unref (tree);
+	gmenu_tree_unref (tree);
 
 	return menu;
 }
 
 static GtkWidget *
-populate_menu_from_directory (GtkWidget         *menu,
-			      MenuTreeDirectory *directory)
+populate_menu_from_directory (GtkWidget          *menu,
+			      GMenuTreeDirectory *directory)
 {	
-	GSList   *subdirs;
-	GSList   *entries;
 	GSList   *l;
+	GSList   *items;
 	gboolean  add_separator;
 
 	add_separator = (GTK_MENU_SHELL (menu)->children != NULL);
 
-	subdirs = menu_tree_directory_get_subdirs (directory);
-	for (l = subdirs; l; l = l->next) {
-		MenuTreeDirectory *subdir = l->data;
+	items = gmenu_tree_directory_get_contents (directory);
 
-		if (add_separator) {
+	for (l = items; l; l = l->next) {
+		GMenuTreeItem *item = l->data;
+
+		if (add_separator ||
+		    gmenu_tree_item_get_type (item) == GMENU_TREE_ITEM_SEPARATOR) {
 			add_menu_separator (menu);
 			add_separator = FALSE;
 		}
 
-		create_submenu (menu, subdir);
+		switch (gmenu_tree_item_get_type (item)) {
+		case GMENU_TREE_ITEM_DIRECTORY:
+			create_submenu (menu, GMENU_TREE_DIRECTORY (item), NULL);
+			break;
 
-		menu_tree_directory_unref (subdir);
-	}
-	g_slist_free (subdirs);
+		case GMENU_TREE_ITEM_ENTRY:
+			create_menuitem (menu, GMENU_TREE_ENTRY (item), NULL);
+			break;
 
-	entries = menu_tree_directory_get_entries (directory);
-	for (l = entries; l; l = l->next) {
-		MenuTreeEntry *entry = l->data;
+		case GMENU_TREE_ITEM_SEPARATOR :
+			/* already added */
+			break;
 
-		if (add_separator) {
-			add_menu_separator (menu);
-			add_separator = FALSE;
+		case GMENU_TREE_ITEM_ALIAS:
+			create_menuitem_from_alias (menu, GMENU_TREE_ALIAS (item));
+			break;
+
+		case GMENU_TREE_ITEM_HEADER:
+			create_header (menu, GMENU_TREE_HEADER (item));
+			break;
+
+		default:
+			break;
 		}
 
-		create_menuitem (menu, entry);
-
-		menu_tree_entry_unref (entry);
+		gmenu_tree_item_unref (item);
 	}
-	g_slist_free (entries);
+
+	g_slist_free (items);
 
 	return menu;
 }
@@ -1454,96 +1670,58 @@ setup_menu_item_with_icon (GtkWidget   *item,
 
 	setup_menuitem (item, icon_size, NULL, title, invisible_mnemonic);
 }
-  
-GtkWidget *
-menu_create_action_item (PanelActionButtonType action_type)
+
+static void
+main_menu_append (GtkWidget   *main_menu,
+		  PanelWidget *panel)
 {
 	GtkWidget *item;
+	gboolean   add_separator;
+	GList     *children;
 
-	if (panel_action_get_is_disabled (action_type))
-		return NULL;
+	if (!g_object_get_data (G_OBJECT (main_menu),
+				"panel-menu-needs-appending"))
+		return;
 
-	item = gtk_image_menu_item_new ();
-        setup_menu_item_with_icon (item,
-				   panel_menu_icon_get_size (),
-				   panel_action_get_icon_name (action_type),
-				   panel_action_get_stock_icon (action_type),
-				   panel_action_get_text (action_type),
-				   TRUE);
+	g_object_set_data (G_OBJECT (main_menu),
+			   "panel-menu-needs-appending", NULL);
 
-	gtk_tooltips_set_tip (panel_tooltips,
-			      item,
-			      panel_action_get_tooltip (action_type),
-			      NULL);
+	children = gtk_container_get_children (GTK_CONTAINER (main_menu));
 
-	g_signal_connect (item, "activate",
-			  panel_action_get_invoke (action_type), NULL);
-	g_signal_connect (G_OBJECT (item), "button_press_event",
-			  G_CALLBACK (menu_dummy_button_press_event), NULL);
-	setup_internal_applet_drag (item, action_type);
+	add_separator = FALSE;
+	if (children != NULL) {
+		while (children->next != NULL)
+			children = children->next;
+		add_separator = !GTK_IS_SEPARATOR (GTK_WIDGET (children->data));
+	}
 
-	return item;
+	if (add_separator)
+		add_menu_separator (main_menu);
+
+	item = panel_place_menu_item_new (TRUE);
+	panel_place_menu_item_set_panel (item, panel);
+	gtk_menu_shell_append (GTK_MENU_SHELL (main_menu), item);
+	gtk_widget_show (item);
+
+	item = panel_desktop_menu_item_new (TRUE, FALSE);
+	panel_desktop_menu_item_set_panel (item, panel);
+	gtk_menu_shell_append (GTK_MENU_SHELL (main_menu), item);
+	gtk_widget_show (item);
+
+	panel_menu_items_append_lock_logout (main_menu);
 }
 
 GtkWidget *
 create_main_menu (PanelWidget *panel)
 {
 	GtkWidget *main_menu;
-	GtkWidget *applications_menu;
-	GtkWidget *menuitem;
 
-	main_menu = create_empty_menu ();
+	main_menu = create_applications_menu ("applications.menu", NULL);
 	g_object_set_data (G_OBJECT (main_menu), "menu_panel", panel);
+	/* FIXME need to update the panel on parent_set */
 
-	menuitem = gtk_image_menu_item_new ();
-	setup_menu_item_with_icon (menuitem,
-				   panel_menu_icon_get_size (),
-				   "gnome-applications",
-				   NULL,
-				   _("Applications"),
-				   TRUE);
-	gtk_menu_shell_append (GTK_MENU_SHELL (main_menu), menuitem);
-
-	applications_menu = create_applications_menu ("applications.menu",
-						      NULL);
-	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem), applications_menu);
-
-	add_menu_separator (main_menu);
-
-	menuitem = menu_create_action_item (PANEL_ACTION_RUN);
-	if (menuitem != NULL)
-		gtk_menu_shell_append (GTK_MENU_SHELL (main_menu), menuitem);
-
-	if (panel_is_program_in_path  ("gnome-search-tool")) {
-		menuitem = menu_create_action_item (PANEL_ACTION_SEARCH);
-		if (menuitem != NULL)
-			gtk_menu_shell_append (GTK_MENU_SHELL (main_menu),
-					       menuitem);
-	}
-
-	panel_recent_append_documents_menu (main_menu);
-
-	add_menu_separator (main_menu);
-
-	if (panel_is_program_in_path ("gnome-screenshot")) {
-		menuitem = menu_create_action_item (PANEL_ACTION_SCREENSHOT);
-		if (menuitem != NULL)
-			gtk_menu_shell_append (GTK_MENU_SHELL (main_menu),
-					       menuitem);
-
-		add_menu_separator (main_menu);
-	}
-
-	if (panel_is_program_in_path  ("xscreensaver")) {
-		menuitem = menu_create_action_item (PANEL_ACTION_LOCK);
-		if (menuitem != NULL)
-			gtk_menu_shell_append (GTK_MENU_SHELL (main_menu),
-					       menuitem);
-	}
-
-	menuitem = menu_create_action_item (PANEL_ACTION_LOGOUT);
-	if (menuitem != NULL)
-		gtk_menu_shell_append (GTK_MENU_SHELL (main_menu), menuitem);
+	g_signal_connect (main_menu, "show",
+			  G_CALLBACK (main_menu_append), panel);
 
 	return main_menu;
 }
@@ -1563,15 +1741,19 @@ find_in_load_list (GtkWidget *image)
 static void
 image_menu_shown (GtkWidget *image, gpointer data)
 {
-	IconToLoad *icon = data;
+	IconToLoad *new_icon;
+	IconToLoad *icon;
+	
+	icon = (IconToLoad *) data;
 
 	/* if we've already handled this */
 	if (gtk_image_get_storage_type (GTK_IMAGE (image)) != GTK_IMAGE_EMPTY)
 		return;
 
 	if (find_in_load_list (image) == NULL) {
-		icons_to_load = g_list_append (icons_to_load,
-					       icon_to_load_copy (icon));
+		new_icon = icon_to_load_copy (icon);
+		new_icon->icon_theme = gtk_icon_theme_get_for_screen (gtk_widget_get_screen (image));
+		icons_to_load = g_list_append (icons_to_load, new_icon);
 	}
 	if (load_icons_id == 0)
 		load_icons_id = g_idle_add (load_icons_handler, NULL);
@@ -1643,14 +1825,14 @@ panel_menu_key_press_handler (GtkWidget   *widget,
 	    (event->state & gtk_accelerator_get_default_mod_mask ()) == GDK_SHIFT_MASK)) {
 		GtkMenuShell *menu_shell = GTK_MENU_SHELL (widget);
 
-		retval = TRUE;
 		if (menu_shell->active_menu_item &&
 		    GTK_MENU_ITEM (menu_shell->active_menu_item)->submenu == NULL) {
 			GdkEventButton bevent;
 
 			bevent.button = 3;
 			bevent.time = GDK_CURRENT_TIME;
-			show_item_menu (menu_shell->active_menu_item, &bevent);
+			retval = show_item_menu (menu_shell->active_menu_item,
+						 &bevent);
 		}
 		
 	}
